unit Freq_Cnt;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  d2xxunit, Dialogs, Menus, StdCtrls, ExtCtrls, MorphPrgDllUnit,
  Frame7Seg;

type
  TFreq_Cnt_Top = class(TForm)
    Panel1: TPanel;
    Timer1: TTimer;
    Timer2: TTimer;
    PopupMenu1: TPopupMenu;
    N1sec1: TMenuItem;
    N500ms1: TMenuItem;
    N100ms1: TMenuItem;
    N10ms1: TMenuItem;
    Auto1: TMenuItem;
    N1: TMenuItem;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    N1ms1: TMenuItem;
    Label4: TLabel;
    Button1: TButton;
    F7Seg1: TF7Seg;
    F7Seg2: TF7Seg;
    F7Seg3: TF7Seg;
    F7Seg4: TF7Seg;
    F7Seg5: TF7Seg;
    F7Seg6: TF7Seg;
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure N1sec1Click(Sender: TObject);
    procedure N500ms1Click(Sender: TObject);
    procedure N100ms1Click(Sender: TObject);
    procedure N10ms1Click(Sender: TObject);
    procedure Auto1Click(Sender: TObject);
    procedure N1ms1Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function ProgFPGA(DevName, RBFFileName : string) : boolean;
function Get_Freq (window : integer) : integer;

var
  Freq_Cnt_Top: TFreq_Cnt_Top;
  MorphPrg: TMorphPrg;
  clON : TColor = clRed;
  clOFF : TColor = $00000047;

implementation

{$R *.dfm}

var
  Dev_Name : string = '<User_Defined>'; //Defined descriptor (must match EEPROM descriptor)
  Dev_Name1 : string = 'Morph-IC-II'; //Default descriptor (must match EEPROM descriptor)
  Dev_Name2 : string = 'MorphIC-II'; //Default descriptor for the first batch published
  RBF_Name : string = 'freq_cnt_top.rbf';
  Dev_Loaded : boolean;
  PortOpen : boolean;
  match : boolean;
  OutIndex : integer;

{ On Creation Check that Morph-IC-II is attached and load RBF }
procedure Connect_Device;
begin
  Dev_Loaded := ProgFPGA(Dev_Name, RBF_Name);
  if Dev_Loaded = False
  then
  begin
  Dev_Name := Dev_Name1;
  Dev_Loaded := ProgFPGA(Dev_Name, RBF_Name);
  end;
  if Dev_Loaded = False
  then
  begin
  Dev_Name := Dev_Name2;
  Dev_Loaded := ProgFPGA(Dev_Name, RBF_Name);
  end;

  Freq_Cnt_Top.Timer1.Enabled := True;
end;

procedure TFreq_Cnt_Top.FormCreate(Sender: TObject);
begin
  Freq_Cnt_Top.F7Seg1.Shape8.Pen.Color := clON;
  Connect_Device;
end;

function ProgFPGA(DevName, RBFFileName : string) : boolean;
var Status : MP_STATUS;
    mpHandle : Dword;
    DeviceName: TDeviceName;
    SerialNumber: TSerialNumber;
    i : integer;
begin
  ProgFPGA := False;

  if not FileExists(RBFFileName) then
  begin
    MessageDlg('Cannot run Frequency Meter -' + #10#10 +
               'RBF File not found!' + #10 +
               '(Please copy into run directory)', mtError, [mbOK], 0);
    exit;
  end;  

  for i := 0 to MAX_NUM_DEVICE_NAME_CHARS-1 do
  begin
    DeviceName[i] := ' ';
    SerialNumber[i] := ' ';
  end;
  
  Status := MorphPrg.MP_GetMorphICDeviceName(1,@DeviceName,MAX_NUM_DEVICE_NAME_CHARS);

  if Status = MP_SUCCESS then
    Status := MorphPrg.MP_GetMorphICDeviceSerialNumber(DeviceName,0,@SerialNumber,MAX_NUM_SERIAL_NUMBER_CHARS);

  if Status = MP_SUCCESS then
    Status := MorphPrg.MP_OpenSpecifiedMorphICDevice(DeviceName,SerialNumber,@mpHandle);

  if Status = MP_SUCCESS then
    Status := MorphPrg.MP_ProgramMorphICDevice(mpHandle, RBFFileName);

  if (Status = MP_SUCCESS) then
    ProgFPGA := True
  else
    MessageDlg('Cannot run Frequency Meter -' + #10#10 +
               'Morph-IC-II not found!' + #10 +
               '(Please ensure device is attached)', mtError, [mbOK], 0);
end;

function OpenPort(ScName : string) : boolean;
var res : FT_Result;
    NoOfDevs,i,J : integer;
    Name,ScopeName, SerialNo : String;
    done : boolean;
begin
  PortOpen := False;
  OpenPort := False;
  match := False;
  Name := '';
  ScopeName := ScName + ' A';
  res := GetFTDeviceCount;
  if res <> Ft_OK then exit;
  NoOfDevs := FT_Device_Count;
  j := 0;
  if NoOfDevs > 0 then
    repeat
      repeat
        res := GetFTDeviceDescription(J);
        if (res <> Ft_OK) then  J := J + 1;
      until (res = Ft_OK) OR (J=NoOfDevs);
      if res <> Ft_OK then exit;
      done := false;
      i := 1;
      Name := '';
      repeat
        if ORD(FT_Device_String_Buffer[i]) <> 0 then
        begin
          Name := Name + FT_Device_String_Buffer[i];
          OpenPort := TRUE;
        end
        else
        begin
          done := true;
          OpenPort := FALSE;
        end;
        i := i + 1;
      until done;
      J := J + 1;
    until (J = NoOfDevs) or (name=ScopeName);

  if (name = ScopeName) then
  begin
    res := GetFTDeviceSerialNo(J-1);
    if res <> Ft_OK then exit;
    done := false;
    SerialNo := '';
    i := 1;
    repeat
      if ORD(FT_Device_String_Buffer[i]) <> 0 then
        SerialNo := SerialNo + FT_Device_String_Buffer[i]
      else
        done := true;
      Inc(i);
    until done;
    res := Open_USB_Device_By_Serial_Number(SerialNo);
    if res <> Ft_OK then exit;
    OpenPort := true;
    res := Get_USB_Device_QueueStatus;
    if res <> Ft_OK then exit;
    PortOpen := True;
    Set_USB_Device_LatencyTimer(1); //set to 1ms
  end
  else
  begin
    MessageDlg(ScopeName+' Not Found ... '+ name, mtError, [mbOk], 0);
    OpenPort := false;
  end;
  /////////
  if name = ScopeName
  then
  match := true;
  ///////////
end;

procedure ClosePort;
begin
  if PortOpen then
    Close_USB_Device;
  PortOpen := False;
  Freq_Cnt_Top.Button1.Visible := True;
end;

procedure AddToBuffer(I : integer);
begin
  if PortOpen then
  begin
    FT_Out_Buffer[OutIndex]:= I and $FF;
    OutIndex := OutIndex + 1;
  end;
end;

procedure SendBytes(NumberOfBytes : integer);
var i : integer;
begin
  if PortOpen then
  begin
    i := Write_USB_Device_Buffer(NumberOfBytes);
    OutIndex := OutIndex - i;
  end;
  if (FT_IO_Status <> FT_OK) then
  begin
    ClosePort;
    OutIndex := 0;
  end;
end;

function ReceiveBuffer : integer;
var i :integer;
    res : FT_Result;
begin
  result := 0;
  if (PortOpen) then
    if FT_IO_Status = FT_OK then
    begin
      res := Get_USB_Device_QueueStatus;
      if res = FT_OK then
      begin
        if (FT_Q_Bytes > 0) then
        begin
          i := Read_USB_Device_Buffer(FT_Q_Bytes);
          result := i;
        end;
      end
      else
        ClosePort;
    end;
end;

{ Timeout if end_time is ms_timeout greater than start_time }
function timeout_ms(start_time, end_time: TDateTime; ms_timeout: integer): boolean;
var
  start_ms, end_ms: integer;   // will allow time diffs of max 20 days!
begin
  start_ms := round(frac(start_time) * 24 * 60 * 60 * 1000);
  end_ms := round(((trunc(end_time) - trunc(start_time)) + frac(end_time)) * 24 * 60 * 60 * 1000);
  timeout_ms := (end_ms - start_ms) >= ms_timeout;
end;

procedure Rotate_Wheel;
begin
  if (Freq_Cnt_Top.Label4.Caption = '-') then
    Freq_Cnt_Top.Label4.Caption := '\'
  else if (Freq_Cnt_Top.Label4.Caption = '\') then
    Freq_Cnt_Top.Label4.Caption := '|'
  else if (Freq_Cnt_Top.Label4.Caption = '|') then
    Freq_Cnt_Top.Label4.Caption := '/'
  else
    Freq_Cnt_Top.Label4.Caption := '-'
end;

function NextChar(var InStr: string) : char;
var TempChar : char;
    i : integer;
    StrLength : integer;
begin
  StrLength := Length(InStr);
  
  if (StrLength > 0) then
  begin
    TempChar := InStr[1];
    // remove char from start of string (two if dec point is present in pos 2)
    if (Length(InStr) > 1) then
      if (InStr[2] = '.') then
        InStr := Copy(InStr, 3, 7)
      else
        InStr := Copy(InStr, 2, 7);
  end
  else
    TempChar := '0';

  if (StrLength = 1) then
    SetLength(InStr, 0);

  Result := TempChar;
end;

procedure Update_Meter(Periods : integer; Window_s : real);
var Freq_Hz : real;
    Freq_KHz : real;
    Freq_MHz : real;
    Freq_Str : string;
    DecPos : integer;
    SegIndex : integer;
    DecPoint : boolean;
begin
  Freq_Hz := (Periods) / (Window_s);
  Freq_KHz := Freq_Hz / 1000;
  Freq_MHz := Freq_KHz / 1000;
  
  if (Freq_MHz >= 1) then
  begin
    Freq_Str := FloatToStr(Freq_MHz);
    Freq_Cnt_Top.Label3.Caption := 'MHz';
  end
  else if (Freq_KHz >= 1) then
  begin
    Freq_Str := FloatToStr(Freq_KHz);
    Freq_Cnt_Top.Label3.Caption := 'KHz';
  end
  else
  begin
    Freq_Str := FloatToStr(Freq_Hz);
    Freq_Cnt_Top.Label3.Caption := 'Hz';
  end;

  // Find decimal point and note position. If no decimal insert one after the
  // length of the string.
  DecPos := Pos('.', Freq_Str);
  if (DecPos = 0) then
    DecPos := Length(Freq_Str)+1;

  // Update each 7 segment display and increment SegIndex after each one
  SegIndex := 1;
  FillSeg7(NextChar(Freq_Str), (SegIndex+1 = DecPos), @Freq_Cnt_Top.F7Seg1);

  Inc(SegIndex);
  FillSeg7(NextChar(Freq_Str), (SegIndex+1 = DecPos), @Freq_Cnt_Top.F7Seg2);

  Inc(SegIndex);
  FillSeg7(NextChar(Freq_Str), (SegIndex+1 = DecPos), @Freq_Cnt_Top.F7Seg3);

  Inc(SegIndex);
  FillSeg7(NextChar(Freq_Str), (SegIndex+1 = DecPos), @Freq_Cnt_Top.F7Seg4);

  Inc(SegIndex);
  FillSeg7(NextChar(Freq_Str), (SegIndex+1 = DecPos), @Freq_Cnt_Top.F7Seg5);

  Inc(SegIndex);
  FillSeg7(NextChar(Freq_Str), (SegIndex+1 = DecPos), @Freq_Cnt_Top.F7Seg6);

end;

{ Function to get number of periods in a predefined window. Window variable is }
{ the command sent to start the measurement:                                   }
{     $B0 -   1 ms }
{     $B1 -  10 ms }
{     $B2 - 100 ms }
{     $B3 - 500 ms }
{     $B4 -   1 s  }
function Get_Freq (window : integer) : integer;
var NoOfBytes : integer;
    NoOfPeriods : integer;
    i : integer;
    done : boolean;
    timeout_start: TDateTime;
    timeout_end : TDateTime;
    timeout : boolean;
begin
  Get_Freq := -1;
  NoOfPeriods := 0;
  AddToBuffer(window);
  SendBytes(1);

  timeout_start := now;
  done := False;
  repeat    
    NoOfBytes := ReceiveBuffer;
    for i := 0 to NoOfBytes-1 do
    begin
      case (FT_In_Buffer[i] and $C0) of
        $00 : NoOfPeriods := FT_In_Buffer[i] and $3F;
        $40 : NoOfPeriods := NoOfPeriods + ((FT_In_Buffer[i] and $3F) SHL 6);
        $80 : NoOfPeriods := NoOfPeriods + ((FT_In_Buffer[i] and $3F) SHL 12);
        $C0 : NoOfPeriods := NoOfPeriods + ((FT_In_Buffer[i] and $3F) SHL 18);
      end;

      done := (FT_In_Buffer[i] and $C0) = $C0;
    end;
    timeout_end := now;
    timeout := timeout_ms(timeout_start, timeout_end, 5000);
    Application.ProcessMessages;
  until done or timeout;

  if done then
    Get_Freq := NoOfPeriods;
end;

{ Exit the program if Morph-IC-II was not loaded for any reason }
{ Otherwise Open Port A for Communicating                    }
procedure TFreq_Cnt_Top.Timer1Timer(Sender: TObject);
begin
  if not Dev_Loaded then
    Freq_Cnt_Top.Button1.Visible := True
//    Freq_Cnt_Top.Close
  else
  begin
    OpenPort(Dev_Name);
    if match = false
    then
    begin
    Dev_Name := Dev_Name1;
    OpenPort(Dev_Name);
    end;
    if match = false
    then
    begin
    Dev_Name := Dev_Name2;
    OpenPort(Dev_Name);
    end;
    ReceiveBuffer;
    Freq_Cnt_Top.Button1.Visible := False;
    Freq_Cnt_Top.Timer2.Enabled := True;
  end;

  Freq_Cnt_Top.Timer1.Enabled := False;
end;

procedure TFreq_Cnt_Top.Timer2Timer(Sender: TObject);
var NoOfPeriods : integer;
    window : real;
    Cmd : integer;
    AutoWindow : boolean;
    timeout : boolean;
begin
  Freq_Cnt_Top.Timer2.Enabled := False;

  // Set command to get Frequncy from Morph-IC-II
  if (Freq_Cnt_Top.N1sec1.Checked) then
    Cmd := $B4
  else if (Freq_Cnt_Top.N500ms1.Checked) then
    Cmd := $B3
  else if (Freq_Cnt_Top.N100ms1.Checked) then
    Cmd := $B2
  else if (Freq_Cnt_Top.N10ms1.Checked) then
    Cmd := $B1
  else
    Cmd := $B0;

  AutoWindow := Freq_Cnt_Top.Auto1.Checked;

  // Send cmd to get Frequency and adjust window until > 1000 periods are found
  repeat
    NoOfPeriods := Get_Freq(Cmd);
    timeout := NoOfPeriods < 0;
    if (NoOfPeriods < 1000) and (AutoWindow) then
      Inc(Cmd);
    Application.ProcessMessages;
  until (NoOfPeriods >= 1000) or (Cmd = $B5) or (not AutoWindow) or (timeout);

  case (Cmd and $F7) of
    $B0 : begin
            window := 0.001;
            Freq_Cnt_Top.Label2.Caption := '1 ms';
          end;
    $B1 : begin
            window := 0.01;
            Freq_Cnt_Top.Label2.Caption := '10 ms';
          end;
    $B2 : begin
            window := 0.1;
            Freq_Cnt_Top.Label2.Caption := '100 ms';
          end;
    $B3 : begin
            window := 0.5;
            Freq_Cnt_Top.Label2.Caption := '500 ms';
          end;
    else  begin
            window := 1;
            Freq_Cnt_Top.Label2.Caption := '1 sec';
          end;
  end;
  
  if (not timeout) then
  begin
    Update_Meter(NoOfPeriods, window);
    Rotate_Wheel;
  end
  else
    Update_Meter(0, window);
  
  Freq_Cnt_Top.Timer2.Enabled := True;
end;

procedure TFreq_Cnt_Top.Auto1Click(Sender: TObject);
begin
  Freq_Cnt_Top.Auto1.Checked := True;
end;

procedure TFreq_Cnt_Top.N1sec1Click(Sender: TObject);
begin
  Freq_Cnt_Top.N1sec1.Checked := True;
end;

procedure TFreq_Cnt_Top.N500ms1Click(Sender: TObject);
begin
  Freq_Cnt_Top.N500ms1.Checked := True;
end;

procedure TFreq_Cnt_Top.N100ms1Click(Sender: TObject);
begin
  Freq_Cnt_Top.N100ms1.Checked := True;
end;

procedure TFreq_Cnt_Top.N10ms1Click(Sender: TObject);
begin
  Freq_Cnt_Top.N10ms1.Checked := True;
end;

procedure TFreq_Cnt_Top.N1ms1Click(Sender: TObject);
begin
  Freq_Cnt_Top.N1ms1.Checked := True;
end;

procedure TFreq_Cnt_Top.Button1Click(Sender: TObject);
begin
  Connect_Device;
end;

end.
